﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows.Input;
using ScrumFactory.Composition.ViewModel;
using ScrumFactory.Composition;
using ScrumFactory.Services;
using System.Configuration;
using ScrumFactory.Composition.View;
using System.Diagnostics;
using System.Deployment.Application;

namespace ScrumFactory.Login.ViewModel {


    /// <summary>
    /// Login view model.
    /// </summary>
    [Export(typeof(LoginViewModel))]
    [Export(typeof(ITopMenuViewModel))]
    public class LoginViewModel : BasePanelViewModel, ITopMenuViewModel, INotifyPropertyChanged {


        private IAuthorizationService authorizator;
        private IEventAggregator aggregator;
        private IBackgroundExecutor executor;
        private IDialogService dialogs;

        private IServerUrl serverUrl;

        private bool signing = false;

        private bool validatingTokenAtServer = false;

        [ImportingConstructor]
        public LoginViewModel(
            [Import(typeof(IEventAggregator))] IEventAggregator aggregator,
            [Import(typeof(IBackgroundExecutor))] IBackgroundExecutor executor,
            [Import(typeof(IAuthorizationService))] IAuthorizationService authorizator,
            [Import(typeof(IDialogService))] IDialogService dialogService,
            [Import] IServerUrl serverUrl,
            [ImportMany(typeof(Services.AuthProviders.IOAuthProvider))] IEnumerable<Services.AuthProviders.IOAuthProvider> providers) {

                this.aggregator = aggregator;
                this.executor = executor;
                this.authorizator = authorizator;
                this.dialogs = dialogService;
                this.serverUrl = serverUrl;

                AllProviders = providers.OrderBy(p =>p.ProviderName);

                aggregator.Subscribe(ScrumFactoryEvent.ShellStarted, TryToSignIn);
                aggregator.Subscribe(ScrumFactoryEvent.NotAuthorizedServerExceptionThrowed, TryToSignIn);
                aggregator.Subscribe(ScrumFactoryEvent.ShowLogin, Show);
                aggregator.Subscribe<string>(ScrumFactoryEvent.ServerArgOnInit, OnSetupServer);
                        
                SignInCommand = new DelegateCommand<Services.AuthProviders.IOAuthProvider>(CanSignIn, SignIn);
                OpenHelpCommand = new DelegateCommand(OpenHelp);
                SetDemoServerCommand = new DelegateCommand(SetDemoServer);

                ServerUrl = Properties.Settings.Default.serverUrl;

        }

        private void TryToSignIn() {            
            SignIn(GetLastSelectedProvider());
        }

        public bool ShowDemoServerButton {
            get {
                // if is debug shows
                if (!ApplicationDeployment.IsNetworkDeployed)
                    return true;

                // if is site deployed show
                if (ApplicationDeployment.CurrentDeployment.UpdateLocation.AbsoluteUri.StartsWith("http://www.scrum-factory.com"))
                    return true;

                return false;
            }
        }

        private void SetDemoServer() {
            ServerUrl = "http://www.scrum-factory.com/demo";
        }

        private void OnSetupServer(string server) {
            this.ServerUrl = server;
            Properties.Settings.Default.Save();
        }

        public IEnumerable<Services.AuthProviders.IOAuthProvider> AllProviders { get; private set; }

        
        

        public string ServerUrl { 
            get {
                return serverUrl.Url;
            }
            set {
                serverUrl.Url = value;
                Properties.Settings.Default.serverUrl = value;
                ScrumFactory.Windows.Helpers.Converters.MemberAvatarUrlConverter.ServerUrl = serverUrl.Url;
                OnPropertyChanged("ServerUrl");
            }
        }

       
        private string titleMessage;
        public string TitleMessage {
            get {
                return titleMessage;
            }
            set {
                titleMessage = value;
                OnPropertyChanged("TitleMessage");
            }
        }

        public bool IsSigning {
            get {
                return signing;
            }
            set {
                signing = value;
                OnPropertyChanged("Signing");
                ((DelegateCommand<ScrumFactory.Services.AuthProviders.IOAuthProvider>)SignInCommand).NotifyCanExecuteChanged();
                aggregator.Publish<bool>(ScrumFactoryEvent.Signing, signing);
            }
        }

        

        private Services.AuthProviders.IOAuthProvider selectedProvider;
        public Services.AuthProviders.IOAuthProvider SelectedProvider {
            get {
                return selectedProvider;
            }
            set {
                selectedProvider = value;                
                OnPropertyChanged("SelectedProvider");
                if(selectedProvider!=null)
                    Properties.Settings.Default.lastProvider = selectedProvider.ProviderName;
                else
                    Properties.Settings.Default.lastProvider = null;
            }
        }

        private void Close() {
            if (!View.IsVisible)
                return;
            dialogs.GoBackSelectedTopMenu();
        }

        private bool ValidateTokenAtServer() {

            

            LoginStatusMessage = null;

            if (SelectedProvider == null || String.IsNullOrEmpty(SelectedProvider.ACCESS_TOKEN))                
                return false;
            
            string token = SelectedProvider.ACCESS_TOKEN;

            // validates token with Scrum Factory Server
            MemberProfile myProfile = null;
            validatingTokenAtServer = true;
            try {
                myProfile = authorizator.SignInMember(SelectedProvider.ProviderName, token);
                if (myProfile==null || myProfile.IsActive == false) {
                    LoginStatusMessage = Properties.Resources.user_blocked_message;
                }
            }
            catch (ScrumFactory.Exceptions.VersionMissmatchException ex) {
                if (ex.IsServerVersionNewer)
                    LoginStatusMessage = Properties.Resources.old_client_version_message;
                else
                    LoginStatusMessage = Properties.Resources.old_server_version_message;
            }
            catch (ScrumFactory.Exceptions.AuthorizationProviderNotSupportedException) {
                LoginStatusMessage = String.Format(Properties.Resources.Authorization_provider_not_supported, SelectedProvider.ProviderName, ServerUrl);
            }
            catch (ScrumFactory.Exceptions.NotFoundException) {
                LoginStatusMessage = Properties.Resources.Login_method_not_found;
            }
            catch (ScrumFactory.Exceptions.NotAuthorizedException) {
            }
            catch (Exception ex) {
                LoginStatusMessage = Properties.Resources.Factory_signin_error + "\n" + ex.Message;
            }

            validatingTokenAtServer = false;

            if (myProfile == null) {                
                return false;                
            }

            Properties.Settings.Default.Save();
            
            OnPropertyChanged("IsMemberSigned");
            OnPropertyChanged("SignedMemberProfile");
            
            return true;

        }

        private bool CanSignIn() {            
            return !IsSigning; 
        }

        private void SignIn(Services.AuthProviders.IOAuthProvider provider) {

            if (validatingTokenAtServer)
                return;

            // if no provider is selected yet, shows the login view model for selection
            if (provider == null) {
                Show();
                return;
            }

            SelectedProvider = provider;            
            TitleMessage = Properties.Resources.Signing;

            // tells everyone that we are signin
            IsSigning = true;
            dialogs.CloseAlertMessage();
            
            executor.StartBackgroundTask(
                () => {


                    // validate access token at server
                    if (ValidateTokenAtServer())
                        return false;

                    // try to refresh token
                    SelectedProvider.RefreshAccesToken();
                    if (ValidateTokenAtServer())
                        return false;
                    
                    // if no error, needs to open auth window
                    if(String.IsNullOrEmpty(LoginStatusMessage))
                        return true;

                    return false;
                    
                },
                AfterSignIn);
        }

        private void AfterSignIn(bool needOpenAuthWindow) {

            // could not use the token, opens the auth Window
            if (needOpenAuthWindow) {
                SelectedProvider.SignOut();
                Show();
                OAuthWindow oauthWindow = new OAuthWindow();
                oauthWindow.ShowDialog(SelectedProvider);
                if (SelectedProvider != null && !String.IsNullOrEmpty(SelectedProvider.ACCESS_TOKEN))
                    SignIn(SelectedProvider);
                else
                    IsSigning = false;

                return;
            }

            
            // sucess!!
            if (authorizator.SignedMemberProfile != null) {
            
                // closes the loginview model, and tells everyone the new signed member
                Close();
                aggregator.Publish<MemberProfile>(ScrumFactoryEvent.SignedMemberChanged, authorizator.SignedMemberProfile);

                // not signing any more
                IsSigning = false;

                return;
            }

            // if could not sign in
            dialogs.ShowAlertMessage(Properties.Resources.Factory_could_not_validate_credentials, LoginStatusMessage, null);

            // not signing any more
            IsSigning = false;


        }

        private void SignOut() {
            authorizator.SignOutMember(SelectedProvider.ProviderName);

            if (SelectedProvider != null) {
                SelectedProvider.SignOut();
                SelectedProvider = null;
                Properties.Settings.Default.Save();
            }

            aggregator.Publish<MemberProfile>(ScrumFactoryEvent.SignedMemberChanged, null);
        }
        
        private string loginStatusMessage;
        public string LoginStatusMessage {
            get {
                return loginStatusMessage;
            }
            set {
                loginStatusMessage = value;
                OnPropertyChanged("LoginStatusMessage");
            }
        }

        public System.Windows.Input.ICommand SignInCommand {
            get;
            set;
        }

        public System.Windows.Input.ICommand OpenHelpCommand {
            get;
            set;
        }

        public System.Windows.Input.ICommand SetDemoServerCommand { get; set; }

        /// <summary>
        /// Gets or sets the view.
        /// </summary>
        /// <value>The view.</value>        
        [Import(typeof(Login))]
        public IView View { get; set; }



        public void Show() {

            

            //if (View.IsVisible)
            //    return;

            try {
                if (authorizator.SignedMemberProfile != null)
                    SignOut();
            }
            catch (Exception) { }

            dialogs.SelectTopMenu(this);            
        }

        private Services.AuthProviders.IOAuthProvider GetLastSelectedProvider() {
            return AllProviders.SingleOrDefault(p => p.ProviderName == Properties.Settings.Default.lastProvider);
            
        }

        private void OpenHelp() {
            Process.Start(new ProcessStartInfo("http://www.scrum-factory.com/FAQ.aspx#cantLogin"));
        }




        public string PanelName {
            get { return Properties.Resources.Sign_in; }
        }

        public int PanelDisplayOrder {
            get { return int.MaxValue; }
        }

        public PanelPlacements PanelPlacement {
            get { return PanelPlacements.Hidden; }
        }

        public string ImageUrl {
            get { return null; }
        }
    }

}
